import argparse
from pathlib import Path
import subprocess

from cryptography import x509
from cryptography.hazmat.primitives import hashes, serialization
from cryptography.hazmat.primitives.serialization import pkcs12
from cryptography.hazmat.primitives.asymmetric import ec, ed25519, padding

MAX_HUMAN_AGE = 365 * 200
def generate():
	print("Please name this aegis. You can call it anything. Frequently people use their legal name.")
	name = input("Name? ")
	print("Generating aegis.")
	subprocess.run([
		"openssl", "req", # PKCS#10 certificate generation utility
		"-new", # Generate a new certificate
		"-newkey", "ec", # Generate a new private key using elliptic-curve (ECDSA or ECDH compatible)
		"-pkeyopt", "ec_paramgen_curve:prime256v1", # Use the prime256v1 CE curve from NIST (P-256)
		"-x509", # Create a self-signed certificate instead of a certificate request.
		"-days", str(MAX_HUMAN_AGE), # Set the validity period to the expected max age of a human
		"-subj", f"/CN={name}", # Add the common name for the persona tied to this aegis
		"-out", "cert.pem", # Generate a self-signed certificate file with a .pem extension
		"-keyout", "key.pem", # Generate an encrypted private key file with a .pem extension
		],
		check=True,)

def sign():
	"Sign some arbitrary data."
	parser = argparse.ArgumentParser()
	parser.add_argument("aegis_key_pem", type=Path, help="The file for the PEM-encoded aegis private key.")
	parser.add_argument("message", type=Path, help="The file containing the message to sign.")
	parser.add_argument("-p", "--password", help="The password to use to open the key file.")
	args = parser.parse_args()

	with open(args.aegis_key_pem, "rb") as f:
		private_key = serialization.load_pem_private_key(
			data=f.read(),
			password=args.password.encode("UTF-8"),
		)
	with open(args.message, "rb") as f:
		data = f.read()
		signature = private_key.sign(
			data,
			ec.ECDSA(hashes.SHA256()),
		)
	with open("signature.bin", "wb") as f:
		f.write(signature)
	print("Wrote signature to signature.bin")

def validate():
	"Validate the signature of some arbitrary data."
	parser = argparse.ArgumentParser()
	parser.add_argument("aegis_cert_pem", type=Path, help="The file for the PEM-encoded aegis public certificate.")
	parser.add_argument("message", type=Path, help="The file containing the message to validate.")
	parser.add_argument("signature", type=Path, help="The file containing the signature to validate.")
	args = parser.parse_args()

	with open(args.aegis_cert_pem, "rb") as f:
		certificate = x509.load_pem_x509_certificate(
			data=f.read(),
		)
	with open(args.message, "rb") as f:
		data = f.read()
	with open(args.signature, "rb") as f:
		signature = f.read()

	key = certificate.public_key()
	key.verify(
		signature=signature,
		data=data,
		signature_algorithm=ec.ECDSA(hashes.SHA256()),
	)
	print("Signature is valid")

